
---------------------------------------------------
-- JS VGA CORE, Version July 2012 rewritten
---------------------------------------------------
--
-- Please do not redistribute !
-- You may take parts of the code an use it for inspiration
-- 
---------------------------------------------------
--
-- Function:
--
-- a mix of several oscilloscope functions
-- including FFT and wave presentations
-- (C) 2011 for www.microcontroller.net
--
-- Adaption for Digilent Spartan 3E Devel Board
--
-- Because of the limited FPGA resources,
-- the code was extremely shrinked and simplified
-- to fit into the device, leading to the following
-- restrictions:
--
-- screen resolution fixed only at 1280 x 1024
-- only 3 waves instead of 16
-- only 8 bit resolution instead of 16
-- only 12 bit FFT instead of 24
-- only 512 steps FFT instead of 4096
-- only read part used in FFT
-- only fixed timing with video frequency 108 MHz
-- only 60 Hz resoution instead of 75Hz
-- lower speed was used to get code synthesized
-- fixed amplification of the waves
-- fixed position of the waves
-- only 2 filters instead of 8
-- only IIR-Filter with non enhanced resolution
-- no FIR-Filter
-- fixed oscilloscope resolution
-- all mouse and knob functions removed
-- FFT is not buffered, memory removed
-- only two simple FFT windows instead of all (Hann, Hamming, Kaiser, )
--
--
-- Please note, that the original design runs on a different
-- platform. For easy adaption everything was removed, which
-- had been too board specific.
--
-- because of rapid shrinking the code might come with some debris
-- of instructions and unused options. Please ignore.
--
-- Several extra FFs were introduced to get the code synthesized,
-- remove them for faster FPGAs
--
--
-- Hints
--
---------------------------------------------------
-- DCM section old

-- 50 x 9 / 5 =  90  *  3 / 2 = 135 worked fine but was too quick
-- 50 x 9 / 5 =  90  *  6 / 5 = 108 does not work however

-- DSM section new
-- a cascaded DCM was used to obtain the 108 MHz
-- 50 / 2.5   =  20  ->  20 * 27 / 5 = 108 does work
----------------------------------------------------
--
-- The FFT's output is not buffered, bcause the memory had to
-- be removed for size issues. Therefore there will be sampling
-- artefacts when the FFT changes rapidly, because it is recaluclated
-- for every new line and thus not consistent. Als the position and
-- width is fixed according to the timing of the 108 MHz.
-- The start impuls determines the sammpling between the two lines
-- in the right aread, and is placed that way, that the FFT appears
-- late in the next line. You can also start it earlier but then
-- you might not see the full FFt output

-- latest change:
-- library functions "fixed point to integer" removed. Therefore
-- old libraries are used. This leads to an artefact with the FFT
-- input data which is not realy "signed". In the representation,
-- the windowed fft input will look strange. The FFT works though.
-- (somehow :-))
--
--
-- CHANGE THE CODE ACCORDING TO YOUR NEEDS
--
-- 





library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_unsigned.all;
  use ieee.numeric_std.all;

library UNISIM;
use UNISIM.Vcomponents.all;



entity jsvgacore_s3e is
port
(
		RESET       : in    std_logic;	-- from push buttom
		CLK_OSC_50  : in    std_logic;	-- osc 50 MHz fixed type in s3e board
		--
		FLASH_OE  	: out   std_logic; 	-- spartan 3e specific
		FLASH_CE  	: out   std_logic;	-- spartan 3e specific
		FLASH_WE  	: out   std_logic;	-- spartan 3e specific
		VIDEO_R     : out   std_logic;
		VIDEO_G     : out   std_logic;
		VIDEO_B     : out   std_logic;
		HSYNC       : out   std_logic;
		VSYNC       : out   std_logic
	);
end jsvgacore_s3e;



architecture a of jsvgacore_s3e is

	signal clk_system		: std_logic;
	signal clk_pixel    	: std_logic; -- hor clock
	
	signal ext_reset		: std_logic;
	signal ext_reset_d1	: std_logic;
   signal ext_reset_d2	: std_logic;
	
	signal video_blank	: std_logic;		-- sig to interconnect moduls
	signal video_hsynch	: std_logic;		-- sig to interconnect moduls
	signal video_vsynch	: std_logic;		-- sig to interconnect moduls	
  	signal video_isynch	: std_logic;		-- sig to interconnect moduls
	
	signal image_data		: std_logic_vector( 4 downto 0) := "00101";		-- the final colour to present
	
	signal video_x			: std_logic_vector(10 downto 0) := (others => '0');	-- the x coordinate of the "beam"
	signal video_y			: std_logic_vector(10 downto 0) := (others => '0');	-- the y coordinate of the "beam"

	-- signals to mark x,y in case of active objects for plotting
   signal gitter			: std_logic;							-- flag for mesh
   signal frame			: std_logic;							-- flga for frame to display
	signal wave0			: std_logic;							-- flag for wave 1
   signal wave1			: std_logic;							-- flag for wave 2
   signal wave2 			: std_logic;							-- flag for wave 3
	signal winplot       : std_logic;							-- flag for window function
	signal fftplot       : std_logic;							-- flag for fft input value
	
	
	signal image_count	: std_logic_vector(33 downto 0) := (others => '0');	-- counts images


	-- signals to create the waves
	signal wave_phase			: std_logic_vector( 9 downto 0) := (others => '0');	-- pseudo phase of the waves to generate
	signal wave_value			: std_logic_vector( 7 downto 0) := (others => '0');	-- pseudo Y = f(x) = f(phase)
	signal wave_value_1	   : std_logic_vector( 7 downto 0) := (others => '0'); 	-- modified value for printing
	
	signal wave_value_2tmp	: std_logic_vector(11 downto 0) := (others => '0');
	signal wave_value_2		: std_logic_vector( 7 downto 0) := (others => '0');	
	signal wave_value_2old	: std_logic_vector( 7 downto 0) := (others => '0');	-- delayed fil
	
	signal wave_value_3tmp	: std_logic_vector(11 downto 0) := (others => '0');
	signal wave_value_3		: std_logic_vector( 7 downto 0) := (others => '0');	
	signal wave_value_3old	: std_logic_vector( 7 downto 0) := (others => '0'); 	-- delayed new

	signal video_y_offset 	: std_logic_vector( 9 downto 0) := (others => '0');	-- y pos of beam

	
	-- signals for windows for FFT
	signal window1       : std_logic_vector(17 downto 0) := (others => '0');		-- parabolic
	signal window2       : std_logic_vector(17 downto 0) := (others => '0');		-- squared parabolic
	signal window        : std_logic_vector(17 downto 0) := (others => '0');		-- used windows
	signal fft_input     : std_logic_vector(21 downto 0) := (others => '0');		-- same size as x_real + 8 bits reso

	-- standard Xilinx FFT signal names
	signal start_do		: std_logic;															-- starting point of FFT-ing
   signal xn_re			: std_logic_vector(11 downto 0) := (others => '0');
	signal xn_im			: std_logic_vector(11 downto 0) := "000000000000";
	signal xn_index		: std_logic_vector( 8 downto 0) := (others => '0');
	signal xk_index		: std_logic_vector( 8 downto 0) := (others => '0');
	signal xk_re			: std_logic_vector(21 downto 0) := (others => '0');
	signal xk_im			: std_logic_vector(21 downto 0) := (others => '0');
	signal xk_full 		: std_logic_vector(35 downto 0) := (others => '0');
	signal xk_prod 		: std_logic_vector(35 downto 0) := (others => '0');
   signal fwd_inv			: std_logic;
   signal fwd_inv_we 	: std_logic;

	-- signals for processing FFt output
	signal xk_re_square	: std_logic_vector(35 downto 0) := (others => '0');
	signal xk_im_square	: std_logic_vector(35 downto 0) := (others => '0');	

	signal xk_im_cut		: std_logic_vector(17 downto 0) := (others => '0');
	signal xk_re_cut		: std_logic_vector(17 downto 0) := (others => '0');	
	
   -- print help signals	
	signal full_vector	: std_logic_vector( 9 downto 0) := (others => '0');	--  last line
	signal fft_value		: std_logic_vector( 9 downto 0) := (others => '0');
	signal fft_value_d1	: std_logic_vector( 9 downto 0) := (others => '0');
	signal fft_value_d2	: std_logic_vector( 9 downto 0) := (others => '0');
	signal ffton			: std_logic;
   signal xn_re_old		: std_logic_vector(11 downto 0) := (others => '0'); 	-- delayed real = fft input

   signal noise_count	: std_logic_vector( 7 downto 0) := (others => '0');	-- counter


---------------------------------------------------------------
--
-- components
--
---------------------------------------------------------------	

	
	
COMPONENT video_signal_generator
port (
    RESET       	: in    std_logic;
	 PIXEL_CLOCK	: in    std_logic;
    --
    VIDEO_X			: out   std_logic_vector(10 downto 0);
    VIDEO_Y    	: out   std_logic_vector(10 downto 0);
	 BLANK       	: out   std_logic;
    HSYNC       	: out   std_logic;
    VSYNC       	: out   std_logic;
	 ISYNC       	: out   std_logic
    );
END COMPONENT;


COMPONENT video_signal_out_mixer
port (
    RESET       	: in   std_logic;
	 PIXEL_CLOCK	: in   std_logic;
    COLOR_VALUE	: in   std_logic_vector (4 downto 0);			-- colour code, see sigmix.vhdl
	 BLANK_I      	: in   std_logic;
    HSYNC_I      	: in   std_logic;
    VSYNC_I      	: in   std_logic;
	 --
	 VIDEO_R			: out  std_logic;
    VIDEO_G    	: out  std_logic;
    VIDEO_B    	: out  std_logic;
    HSYNC_O      	: out  std_logic;
    VSYNC_O      	: out  std_logic
    );
END COMPONENT;

	

COMPONENT fft512
  PORT (
    clk : IN STD_LOGIC;
    start : IN STD_LOGIC;
    xn_re : IN STD_LOGIC_VECTOR(11 DOWNTO 0);
    xn_im : IN STD_LOGIC_VECTOR(11 DOWNTO 0);
    fwd_inv : IN STD_LOGIC;
    fwd_inv_we : IN STD_LOGIC;
    rfd : OUT STD_LOGIC;
    xn_index : OUT STD_LOGIC_VECTOR(8 DOWNTO 0);
    busy : OUT STD_LOGIC;
    edone : OUT STD_LOGIC;
    done : OUT STD_LOGIC;
    dv : OUT STD_LOGIC;
    xk_index : OUT STD_LOGIC_VECTOR(8 DOWNTO 0);
    xk_re : OUT STD_LOGIC_VECTOR(21 DOWNTO 0);
    xk_im : OUT STD_LOGIC_VECTOR(21 DOWNTO 0)
  );
END COMPONENT;


COMPONENT clkmcm108new			-- "cascaded Spartan 6 DCM"  50 MHz : 2,5 -> 20 MHz x 27 = 540 MHz : 5 = 108 MHz
	PORT(
		U1_CLKIN_IN 			: IN std_logic;
		U1_RST_IN 				: IN std_logic;          
		CLKDV_OUT 				: OUT std_logic;
		U1_CLKDV_OUT 			: OUT std_logic;
		U1_CLKIN_IBUFG_OUT 	: OUT std_logic;
		U1_CLK0_OUT 			: OUT std_logic;
		U2_CLKFX_OUT 			: OUT std_logic;
		U2_CLK0_OUT		 		: OUT std_logic;
		U2_LOCKED_OUT 			: OUT std_logic
		);
END COMPONENT;
	
	

begin

  ----------------------------------------------------------------------------------------------------------------------------------
  -- Disable unused components
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  --StrataFLASH must be disabled to prevent it conflicting with the LCD display
  --
	FLASH_OE <= '1';
	FLASH_CE <= '1';
	FLASH_WE <= '1';
  --
  


-- use a double DCM in order to obtain the required frequency


-- this is for 108 MHz clk speed -> 60Hz
clkmcm108: clkmcm108new
port map (
		U1_CLKIN_IN 			=> CLK_OSC_50,			-- given by Spartan 6 Devel Board
		U1_RST_IN 				=> RESET,				-- asynchronous button input
		CLKDV_OUT		 		=> open,
		U1_CLKDV_OUT 			=> open,
		U1_CLKIN_IBUFG_OUT 	=> open ,
		U1_CLK0_OUT 			=> clk_system,			-- anything
		U2_CLKFX_OUT 			=> clk_pixel,			-- must 108 MHz
		U2_CLK0_OUT 			=> open,
		U2_LOCKED_OUT 			=> open
	);

	

-- sig sig generator creates all signals
siggen : video_signal_generator
port map (
		-- in
		RESET				=> ext_reset,
		PIXEL_CLOCK   	=> clk_pixel,
		-- out
		VIDEO_X			=> video_x,				-- the generated X-Pos or all modules / calcs
		VIDEO_Y			=> video_y,				-- the generated Y-Pos or all modules / calcs
		BLANK       	=> video_blank,
		HSYNC         	=> video_hsynch,
		VSYNC         	=> video_vsynch,
		ISYNC         	=> video_isynch		-- "new image"
);


-- this module transforms a colour value and signals to RGB video signals
-- can be left away / simplified when colour interlacing is not desired
sigmix : video_signal_out_mixer
port map (
		-- in
		RESET			=> ext_reset,			-- synched reset
		PIXEL_CLOCK	=> clk_pixel,			-- 108 MHz
		COLOR_VALUE	=> image_data,			-- colour code 
		BLANK_I     => video_blank,
		HSYNC_I     => video_hsynch,
		VSYNC_I     => video_vsynch,
		-- out
		VIDEO_R     => VIDEO_R,				-- should not be registered, otherwise interlacing might fail
		VIDEO_G     => VIDEO_G,
		VIDEO_B     => VIDEO_B,
		HSYNC_O     => HSYNC,
		VSYNC_O     => VSYNC
);



my_fft512 : fft512
PORT MAP (
    clk 				=> clk_pixel,
    start 			=> start_do,
    xn_re 			=> xn_re,
    xn_im 			=> xn_im,
    fwd_inv 		=> fwd_inv,
    fwd_inv_we 	=> fwd_inv_we,
    rfd 				=> open,
    xn_index 		=> xn_index,			-- indes of data input to fft
    busy 			=> open,
    edone 			=> open,
    done 			=> open,
    dv 				=> open,
    xk_index 		=> xk_index,			-- index of freq output from fft
    xk_re 			=> xk_re,
    xk_im 			=> xk_im
  );


  

mk_reset : process(clk_pixel)
begin
	if rising_edge(clk_pixel) then
	
		if (RESET = '1') then
			ext_reset_d2 <= '1';						-- backward synched
		else
			ext_reset_d2 <= '0';
		end if;

		ext_reset_d1 <= ext_reset_d2;		
		ext_reset <= ext_reset_d1;

	end if;
			
end process;


mk_noise : process(clk_system)						-- produce some data which is asynchronous to to video clock
begin
	if rising_edge(clk_system) then
	
		noise_count <= std_logic_vector (unsigned(noise_count) + 1);
		
	end if;
			
end process;




mk_count : process(clk_pixel)
begin
	if rising_edge(clk_pixel) then
	
		if (ext_reset = '1') then
			 image_count <= (others => '0');
			 
		elsif (video_isynch = '1') then				-- count of every "new image" pulse
			 image_count <= std_logic_vector (unsigned(image_count) + 1);
		
		end if;

	end if;
			
end process;




-------------------------------------


mk_wave : process(clk_pixel)		-- produce a dummy wave
begin
 

	if rising_edge(clk_pixel) then
	
		-- create the "phase" for the waves from the x-position of "vga beam" and a variable offset
		-- thus, the presented wave ist not stable but slightly moving

		if (image_count(14) = '1') then				-- switch between running and steady	
			wave_phase <= std_logic_vector (unsigned(video_x(9 downto 0)) + unsigned(image_count(9 downto 0))); -- fast
		else
			wave_phase <= std_logic_vector (unsigned(video_x(9 downto 0)) + unsigned(image_count(11 downto 4))); -- slow
		end if;
		
		-- scramble the bits to get a pattern for waveform 1
		if (image_count(13 downto 11) = "000") then  	-- switch to waveform triangle
			wave_value (0) <= '0';
			wave_value (1) <= wave_phase(1);
			wave_value (2) <= wave_phase(2);
			wave_value (3) <= wave_phase(3);
			wave_value (4) <= wave_phase(4);
			wave_value (5) <= wave_phase(5);
			wave_value (6) <= wave_phase(6);
			wave_value (7) <= wave_phase(7);
			

		-- create waveform 2
		elsif (image_count(13 downto 11) = "001") then  -- switch to waveform "rising square"
			wave_value (0) <= wave_phase(4);
			wave_value (1) <= wave_phase(5);
			wave_value (2) <= wave_phase(6);
			wave_value (3) <= wave_phase(7);
			wave_value (4) <= wave_phase(8);
			wave_value (5) <= wave_phase(5);
			wave_value (6) <= wave_phase(5);
			wave_value (7) <= wave_phase(5);			
	

		-- create waveform 3
		elsif (image_count(13 downto 11) = "010") then  -- switch to waeform "noisy square"
			wave_value (0) <= wave_phase(5);
			wave_value (1) <= wave_phase(4);
			wave_value (2) <= wave_phase(3);
			wave_value (3) <= wave_phase(2);
			wave_value (4) <= wave_phase(1);
			wave_value (5) <= wave_phase(0);
			wave_value (6) <= wave_phase(6);
			wave_value (7) <= wave_phase(6);
			

		-- create waveform 4			
		elsif (image_count(13 downto 11) = "011") then  -- switch to waveform "stairs"

			wave_value (0) <= wave_phase(5);
			wave_value (1) <= wave_phase(5);
			wave_value (2) <= wave_phase(5);
			wave_value (3) <= wave_phase(5);
			wave_value (4) <= wave_phase(4);
			wave_value (5) <= wave_phase(4);
			wave_value (6) <= wave_phase(4);
			wave_value (7) <= wave_phase(4);
		

		elsif (image_count(13 downto 11) = "100") then  -- switch to waveform "quick squares"

			wave_value (0) <= wave_phase(4);
			wave_value (1) <= wave_phase(4);
			wave_value (2) <= wave_phase(4);
			wave_value (3) <= wave_phase(4);
			wave_value (4) <= wave_phase(4);
			wave_value (5) <= wave_phase(4);
			wave_value (6) <= wave_phase(4);
			wave_value (7) <= wave_phase(4);
				

		elsif (image_count(13 downto 11) = "101") then  -- switch to waveform "invers triangle"

			wave_value <= (not wave_phase(7 downto 0));
	

		elsif (image_count(13 downto 11) = "110") then  -- switch to waveform "pseudo sinus"

			if (wave_phase(6) = '1') then
				wave_value <= std_logic_vector (69 + unsigned(wave_phase(5 downto 2)) * ( 15 - unsigned(wave_phase(5 downto 2))) ) & '0';
			else
				wave_value <= std_logic_vector (59 - unsigned(wave_phase(5 downto 2)) * ( 15 - unsigned(wave_phase(5 downto 2))) ) & '0';
			end if;
			
	   else		-- use sampled asynchronous "noise" data
		
			wave_value (0) <= noise_count(7);
			wave_value (1) <= noise_count(6);
			wave_value (2) <= noise_count(5);
			wave_value (3) <= noise_count(4);
			wave_value (4) <= noise_count(3);
			wave_value (5) <= noise_count(2);
			wave_value (6) <= noise_count(1);
			wave_value (7) <= noise_count(0);
			
		end if;


		-- prepare wave value 0, must be small enough to show up, because has high amplitude
		wave_value_1 <= wave_value;	
		
		-- filter the pattern with an IIR-Filter to get a more smoothed wave:
		-- Ynew = Yold * k  + newValue * (1-k)   k = 15/16
		wave_value_2tmp <= std_logic_vector (unsigned(wave_value_2tmp) - unsigned(wave_value_2tmp(11 downto 4)) + unsigned(wave_value_1));

		-- size the filtered value to get wave 2
		wave_value_2 <= std_logic_vector (unsigned(wave_value_2tmp(11 downto 4)) + 0); 

		-- filter once more to get an even more smoothed wave
		wave_value_3tmp <= std_logic_vector (unsigned(wave_value_3tmp) - unsigned(wave_value_3tmp(11 downto 4)) + unsigned(wave_value_2));

		-- create sized new value
		wave_value_3 <= std_logic_vector (unsigned(wave_value_3tmp(11 downto 4)) + 0 );
		

	end if;	

end process;






-------------------------------------------------------------------------------------
-- perform fft
-------------------------------------------------------------------------------------

mk_fft : process(clk_pixel)		-- produce a dummy wave
begin

	if rising_edge(clk_pixel) then

		if unsigned(video_x) = 637 then 				-- start impulse for fft to fit mesh
																-- the fft's output apears the next line
			start_do <= '1';
		else
			start_do <= '0';
		end if;

		
		-- make a phase and window for windowing
		-- my standard pseudo sinus function
		-- see the article on www.mikrocontroller.net "Digitale Sinusfunktion"
		window1 <= std_logic_vector (unsigned(xn_index(8 downto 0)) * (511- unsigned(xn_index(8 downto 0))) );
		-- produce a soft cosinus like window, similar to the "von Hann" - window
		window2 <= std_logic_vector (unsigned(window1(17 downto 9)) * unsigned(window1(17 downto 9)));
		
		if (image_count(7) = '1') then		-- switch the windows
			window <= window1;
		else
			window <= window2 (15 downto 0) & "00";		
		end if;
	
		
		if (image_count(9) = '1') then				-- switch FFT input twice in between the 4 waves
																-- feed fft with filtered or double filtered value
			fft_input <= std_logic_vector ((signed('0' & wave_value_3tmp(11 downto 0))-2048) * signed('0' & window(17 downto 8)));
			--fft_input <= std_logic_vector ( unsigned(wave_value_3tmp(11 downto 0)) * unsigned(window(17 downto 8)));
			-- zwo bits overloaded
		else
			fft_input <= std_logic_vector ((signed('0' & wave_value_2tmp(11 downto 0))-2048) * signed('0' & window(17 downto 8)));
			--fft_input <= std_logic_vector ( unsigned(wave_value_2tmp(11 downto 0)) * unsigned(window(17 downto 8)));

		end if;

		xn_re	<= std_logic_vector (unsigned(fft_input (21 downto 10)) - 0);
		
		xn_im <= (others => '0');


		-- FFT core acts here


		-- limit the output by cutting extreme values to save resources
		if (xk_re > "0000111111111111111111") then 
			xk_re_cut <= "111111111111111111";
		else
			xk_re_cut <= xk_re(17 downto 0);
		end if;

		if (xk_im > "0000111111111111111111") then 
			xk_im_cut <= "111111111111111111";
		else
			xk_im_cut <= xk_im(17 downto 0);
		end if;


		-- calculate squares in an extra step for this slow fpga
		-- 18 bit x 18 bit = 36 bit
		-- attention: core provides signed!
		xk_re_square <= std_logic_vector(signed(xk_re_cut) * signed(xk_re_cut));
		xk_im_square <= std_logic_vector(signed(xk_im_cut) * signed(xk_im_cut));

		-- create the full FFT output  y=  Re*Re + Im*Im
		-- 36 bit + 36 bit = 36 bit
		xk_prod <= std_logic_vector(unsigned(xk_re_square) + unsigned(xk_im_square));
		xk_full <= xk_prod;
		
		-- produce "logarithmic" scale and also shun square root calculation of xk_full
		
		full_vector	<= "1111111110"; 	-- start fft from this graphical position
		
		-- now, perform a manual logarithmic operation
		
			if (xk_full (35) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (36 * 8));
		elsif (xk_full (34) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (35 * 8));
		elsif (xk_full (33) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (34 * 8));
		elsif (xk_full (32) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (33 * 8));
		elsif (xk_full (31) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (32 * 8));
		elsif (xk_full (30) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (31 * 8));
		elsif (xk_full (29) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (30 * 8));
		elsif (xk_full (28) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (29 * 8));
		elsif (xk_full (27) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (28 * 8));
		elsif (xk_full (26) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (27 * 8));
		elsif (xk_full (25) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (26 * 8));
		elsif (xk_full (24) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (25 * 8));
		elsif (xk_full (23) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (24 * 8));
		elsif (xk_full (22) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (23 * 8));
		elsif (xk_full (21) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (22 * 8));
		elsif (xk_full (20) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (21 * 8));
		elsif (xk_full (19) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (20 * 8));
		elsif (xk_full (18) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (19 * 8));
		elsif (xk_full (17) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (18 * 8));
		elsif (xk_full (16) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (17 * 8));
		elsif (xk_full (15) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (16 * 8));
		elsif (xk_full (14) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (15 * 8));
		elsif (xk_full (13) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (14 * 8));
		elsif (xk_full (12) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (13 * 8));
		elsif (xk_full (11) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (12 * 8));
		elsif (xk_full (10) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (11 * 8));
		elsif (xk_full ( 9) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - (10 * 8));
		elsif (xk_full ( 8) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 9 * 8));
		elsif (xk_full ( 7) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 8 * 8));
		elsif (xk_full ( 6) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 7 * 8));
		elsif (xk_full ( 5) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 6 * 8));
		elsif (xk_full ( 4) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 5 * 8));
		elsif (xk_full ( 3) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 4 * 8));
		elsif (xk_full ( 2) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 3 * 8));
		elsif (xk_full ( 1) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 2 * 8));
		elsif (xk_full ( 0) = '1') then fft_value <= std_logic_vector(unsigned(full_vector) - ( 1 * 8));		
		else
			fft_value <=  std_logic_vector(unsigned(full_vector) -  1);
		end if;

		-- register for speed
		fft_value_d1 <= fft_value;
		fft_value_d2 <= fft_value_d1;


		-- print vertical line as fft window, when scanning takes place
				
		if (unsigned(xn_index) = 2) then
			ffton <= '1';
			
		elsif (unsigned(xn_index) = 509) then
			ffton <= '1';

		--elsif (unsigned(xk_index) = 1) then			-- print beginning out output
			--ffton <= '1';

		--elsif (unsigned(xk_index) = 511) then		-- print end of output
			--ffton <= '1';
			
		-- print fft values
		elsif (unsigned(video_y) >= unsigned (fft_value_d2) ) then -- below value?
--			if (unsigned(video_y) < unsigned (full_vector) ) then -- still above bottom limit
--				ffton <= '1';
--				
--			else
--				ffton <= '0';
--			end if;
			
			ffton <= '1';			
		else
			ffton <= '0';
		end if;
			
		
	end if;	

end process;


xn_im <= (others => '0');
fwd_inv <= '0';
fwd_inv_we <= '1';



----------------------------------
-- draw waves on screen in real time
----------------------------------

mk_draw : process(clk_pixel)						-- "print" the waves by marking the pixels
begin

	if rising_edge(clk_pixel) then
	
		video_y_offset <= "0110000000";		-- define position of zero line in screen


		-- printing option 0 - create dots only for waves
		if (unsigned(video_y) = (unsigned(video_y_offset) - unsigned (wave_value_1)) ) then
			wave0 <= '1';
		else
			wave0 <= '0';		
		end if;	
		
		
		-- drawing method 2 - create lines for waves 1 + 2

		wave_value_2old <= wave_value_2;				-- use the former value as starting point

		if    (unsigned(video_y) <= (unsigned(video_y_offset) - unsigned (wave_value_2)) ) then
			if (unsigned(video_y) >= (unsigned(video_y_offset) - unsigned (wave_value_2old)) ) then
				wave1 <= '1';
			else
				wave1 <= '0';		
			end if;

		elsif (unsigned(video_y) >= (unsigned(video_y_offset) - unsigned (wave_value_2)) ) then
			if (unsigned(video_y) <= (unsigned(video_y_offset) - unsigned (wave_value_2old)) ) then
				wave1 <= '1';
			else
				wave1 <= '0';		
			end if;
			
		else
			wave1 <= '0';		
		end if;	

		
		wave_value_3old <= wave_value_3;
		
		if    (unsigned(video_y) <= (unsigned(video_y_offset) - unsigned (wave_value_3)) ) then
			if (unsigned(video_y) >= (unsigned(video_y_offset) - unsigned (wave_value_3old)) ) then
				wave2 <= '1';
			else
				wave2 <= '0';		
			end if;

		elsif (unsigned(video_y) >= (unsigned(video_y_offset) - unsigned (wave_value_3)) ) then
			if (unsigned(video_y) <= (unsigned(video_y_offset) - unsigned (wave_value_3old)) ) then
				wave2 <= '1';
			else
				wave2 <= '0';		
			end if;
			
		else
			wave2 <= '0';		
		end if;	
		
		-- print fft window function
		if (unsigned(video_y) = (767 - unsigned (window(17 downto 8))) ) then
			winplot <= '1';
		else
			winplot <= '0';		
		end if;
		

		-- print the fft input value, in fact it is the real only
		
		xn_re_old <= xn_re;
		
		if    (unsigned(video_y) <= (767 - unsigned (xn_re    (11 downto 1))) ) then
			if (unsigned(video_y) >= (767 - unsigned (xn_re_old(11 downto 1))) ) then
				fftplot <= '1';
			else
				fftplot <= '0';		
			end if;

		elsif (unsigned(video_y) >= (767 - unsigned (xn_re    (11 downto 1))) ) then
			if (unsigned(video_y) <= (767 - unsigned (xn_re_old(11 downto 1))) ) then
				fftplot <= '1';
			else
				fftplot <= '0';		
			end if;
			
		else
			fftplot <= '0';		
		end if;	
		
	
	end if;

end process;




-- 
mk_frame : process(video_x,video_y)
begin
			if (unsigned(video_x) <=   1) then
			frame <= '1';
		elsif (unsigned(video_x) >= 1278) then
			frame <= '1';
		elsif (unsigned(video_y) <=   1) then
			frame <= '1';
		elsif (unsigned(video_y) >= 1022) then
			frame <= '1';
		else
			frame <= '0';
		end if;

end process;


mk_gitter : process(video_x,video_y)
begin

		-- generate a mesh pattern similar to an oscilloscope screen
			if ((unsigned(video_x(6 downto 0)) - 0 ) =    0) then
			gitter <= video_x(0) xor video_y(0);
		elsif ((unsigned(video_x(6 downto 0)) - 0 ) =   127) then
			gitter <= video_x(0) xor video_y(0);
		elsif ((unsigned(video_y(6 downto 0)) + 0 ) =    0) then
			gitter <= video_x(0) xor video_y(0);
		elsif ((unsigned(video_y(6 downto 0)) + 0 ) =  127) then
			gitter <= video_x(0) xor video_y(0);
			
		-- generate the mid lines with rulers	
		elsif ( ((unsigned(video_x(4 downto 0)) - 0 ) = 0) or ((unsigned(video_x(4 downto 0)) - 0 ) = 15) ) then
			if ((unsigned(video_y) >= 507) and (unsigned(video_y) <= 517)) then
				gitter <= '1';
			else
				gitter <= '0';			
			end if;

		elsif ( ((unsigned(video_y(4 downto 0)) + 0 ) = 0) or ((unsigned(video_y(4 downto 0)) + 0 ) = 15) ) then
			if ((unsigned(video_x) >= 634) and (unsigned(video_x) <= 645)) then
				gitter <= '1';
			else
				gitter <= '0';			
			end if;
			
			
		else
			gitter <= '0';

	end if;
end process;



----------------------------------------------------
-- mix the objects and define their colours
----------------------------------------------------
mk_cols  : process (clk_pixel)
begin
	if rising_edge(clk_pixel) then
		
			if (frame = '1') then
			image_data <= "11110";		-- frame outside = code "white"

		elsif (gitter = '1') then
			image_data <= "00110";	--    black / green!    00110 = green + green
			
		elsif (wave0 = '1') then
			image_data <= "11110";	-- 	white
			
		elsif (wave1 = '1') then
			image_data <= "00100";	--		yellow 

		elsif (wave2 = '1') then
			image_data <= "01000";	-- 	greenblue 

		elsif (ffton = '1') then
			if (image_count(9) = '0') then	-- print fft values like colour of the related wave 
				image_data <= "00100";	-- like wave 1	
			else
				image_data <= "01000";	-- like wave 2
			end if;

		elsif (winplot = '1') then		-- plot the window for fft
			image_data <= "11000";		-- red/white

		elsif (fftplot = '1') then		-- fft data
			image_data <= "11100";		--	white/blue		
	
			
		else
			image_data <= "00000";		-- black = background
			
		end if;
		
	end if;
	
end process;



	  
end a;
