------------------------------------------
--
-- video signal out mixer and colour interpreter
--
------------------------------------------

library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_unsigned.all;
  use ieee.numeric_std.all;
  
--  library UNISIM;
--  use UNISIM.Vcomponents.all;

entity video_signal_out_mixer is
port (
    RESET       	: in   std_logic;
	 PIXEL_CLOCK	: in   std_logic;
    COLOR_VALUE	: in   std_logic_vector (4 downto 0);
	 BLANK_I      	: in   std_logic;
    HSYNC_I      	: in   std_logic;
    VSYNC_I      	: in   std_logic;
	 --
	 VIDEO_R			: out  std_logic;
    VIDEO_G    	: out  std_logic;
    VIDEO_B    	: out  std_logic;
    HSYNC_O      	: out  std_logic;
    VSYNC_O      	: out  std_logic
    );
end;


architecture a of video_signal_out_mixer is

  signal color_code			: std_logic_vector( 5 downto 0) := "000000";		-- hw colour to toggle
  signal toggle				: std_logic_vector( 1 downto 0) := "00";
  
  signal v_sync_d1			: std_logic := '0';
  signal h_sync_d1			: std_logic := '0';
  signal v_sync_d2			: std_logic := '0';
  signal h_sync_d2			: std_logic := '0';
  signal v_sync_d3			: std_logic := '0';
  signal h_sync_d3			: std_logic := '0';
  signal v_sync_d4			: std_logic := '0';
  signal h_sync_d4			: std_logic := '0';
  


begin


---------------------------------------------
-- create colour information from colour index
----------------------------------------------

mk_colours  : process (PIXEL_CLOCK)
begin
	 if rising_edge(PIXEL_CLOCK) then
	 
		if (BLANK_I = '1') then color_code <= "000000";					-- use blank here
	
		elsif (COLOR_VALUE = "00000") then color_code <= "000000";	-- black
		elsif (COLOR_VALUE = "00001") then color_code <= "000001";	-- black/red
		elsif (COLOR_VALUE = "00010") then color_code <= "001001";	-- red  /red
		elsif (COLOR_VALUE = "00011") then color_code <= "001011";	-- red  /yellow
		elsif (COLOR_VALUE = "00100") then color_code <= "011011";	-- yellow
		elsif (COLOR_VALUE = "00101") then color_code <= "011010";	-- yellow/green
		elsif (COLOR_VALUE = "00110") then color_code <= "010010";	-- green
		elsif (COLOR_VALUE = "00111") then color_code <= "010110";	-- green/cyan
		elsif (COLOR_VALUE = "01000") then color_code <= "110110";	-- cyan
		elsif (COLOR_VALUE = "01001") then color_code <= "110100";	-- cyan/blue
		elsif (COLOR_VALUE = "01010") then color_code <= "100100";	-- blue
		elsif (COLOR_VALUE = "01011") then color_code <= "100101";	-- blue/purple
		elsif (COLOR_VALUE = "01100") then color_code <= "101101";	-- purple
		elsif (COLOR_VALUE = "01101") then color_code <= "101001";	-- purple/red
		elsif (COLOR_VALUE = "01110") then color_code <= "000000";	-- black
		elsif (COLOR_VALUE = "01111") then color_code <= "000000";	-- 
		elsif (COLOR_VALUE = "10000") then color_code <= "000001";	-- 1
		elsif (COLOR_VALUE = "10001") then color_code <= "000011";	-- black / yellow
		elsif (COLOR_VALUE = "10010") then color_code <= "000010";	-- black / green
		elsif (COLOR_VALUE = "10011") then color_code <= "000110";	-- 6
		elsif (COLOR_VALUE = "10100") then color_code <= "000100";	-- 4
		elsif (COLOR_VALUE = "10101") then color_code <= "000101";	-- 5
		elsif (COLOR_VALUE = "10110") then color_code <= "000000";	-- black / black
		elsif (COLOR_VALUE = "10111") then color_code <= "111000";	-- 
		elsif (COLOR_VALUE = "11000") then color_code <= "111001";	-- 1
		elsif (COLOR_VALUE = "11001") then color_code <= "111011";	-- 3
		elsif (COLOR_VALUE = "11010") then color_code <= "111010";	-- 2
		elsif (COLOR_VALUE = "11011") then color_code <= "111110";	-- 6
		elsif (COLOR_VALUE = "11100") then color_code <= "111100";	-- white blue
		elsif (COLOR_VALUE = "11101") then color_code <= "111101";	-- 5
		elsif (COLOR_VALUE = "11110") then color_code <= "111111";	-- 
		 else                              color_code <= "111111";	-- white
		end if;
		
	end if;
	
end process;



---------------------
-- asynch part
---------------------

-- here an asynchronous clock will be created toggling the two colour vectors


-- generate a toggling clock impulse, it is self running
  p_toggle : process(RESET, toggle)
  begin
    if (RESET = '1') then
		toggle <= "00";
    else
		toggle <= toggle + "01" after 1111 ps;
    end if;
  end process;
  
  
  -- here, the toogle flaf switches between two possible colour configurations of rgb
  -- thus, on the screen one will observe a mixture of both
  p_toggle_cols : process(toggle, color_code)
  begin
	 if (toggle = "00" or toggle = "10") then
		 VIDEO_R <= color_code(0);
		 VIDEO_G <= color_code(1);
		 VIDEO_B <= color_code(2);
	 else
		 VIDEO_R <= color_code(3);
		 VIDEO_G <= color_code(4);
		 VIDEO_B <= color_code(5);
	 end if;
	
  end process; 

  
------------------
--synch delay
------------------

p_synch_mux : process(PIXEL_CLOCK)
begin
	if rising_edge (PIXEL_CLOCK) then
	
		if (RESET = '1') then
			HSYNC_O <= '1';
			VSYNC_O <= '1';
			
			h_sync_d1 <= '1';
			v_sync_d1 <= '1';

			h_sync_d2 <= '1';
			v_sync_d2 <= '1';

			h_sync_d3 <= '1';
			v_sync_d3 <= '1';
			
			h_sync_d4 <= '1';
			v_sync_d4 <= '1';
			
			
		else
		
			-- perform delay to compensate calculation latency
			h_sync_d1 <= HSYNC_I;
			v_sync_d1 <= VSYNC_I;
			
			h_sync_d2 <= h_sync_d1;
			v_sync_d2 <= v_sync_d1;

			h_sync_d3 <= h_sync_d2;
			v_sync_d3 <= v_sync_d2;

			h_sync_d4 <= h_sync_d3;
			v_sync_d4 <= v_sync_d3;
			
			HSYNC_O <= h_sync_d3;
			VSYNC_O <= v_sync_d3;
			
			--HSYNC_O <= h_sync_d3;
			--VSYNC_O <= v_sync_d3;
			
		end if;		

	end if;

end process;
  
end architecture a;

